library(DBI)
library(odbc)
library(glue)

test_that("connect_to_db works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_true(identical(class(connect_to_db(db = "test_datalibaba", user = "test", ecoSQL = FALSE))[[1]], "PostgreSQLConnection"))

    con<- DBI::dbConnect(DBI::dbDriver("PostgreSQL"),
                         dbname="test_datalibaba",
                         host=Sys.getenv("server"),
                         port=Sys.getenv("port"),
                         user=Sys.getenv("user_test"),
                         password=Sys.getenv("pwd_test"))
    expect_equal(
      str(connect_to_db(db = "test_datalibaba", user = "test", ecoSQL = FALSE)),
      str(con)
    )

    expect_true(identical(class(connect_to_db(db = "test_datalibaba", user = "test"))[[1]], "PqConnection"))
    con<- DBI::dbConnect(RPostgres::Postgres(),
                         dbname="test_datalibaba",
                         host=Sys.getenv("server"),
                         port=Sys.getenv("port"),
                         user=Sys.getenv("user_test"),
                         password=Sys.getenv("pwd_test"))
    expect_equal(
      str(connect_to_db(db = "test_datalibaba", user = "test")),
      str(con)
    )
  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("connect_to_dbi works", {
  listdriver<-odbc::odbcListDrivers()
  print("test connect_to_dbi : list driver odbc")
  print(listdriver)
  Sys.setenv(port="5432")
  Sys.setenv(server="127.0.0.1")
  Sys.setenv(user_test="runner")
  Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))
  patterndriver <-"^Postgre.*UNICODE|Unicode.*"
  drv_unicode_postgres <- listdriver$name[grep(patterndriver, listdriver$name)][1]
  con_chaine <- glue::glue("Driver={",drv_unicode_postgres,"};Server=$Sys.getenv('server')$;Database=test_datalibaba;Uid=$Sys.getenv('user_test')$;Pwd=$Sys.getenv('pwd_test')$;Port=$Sys.getenv('port')$;",
                           .open = "$",
                           .close= "$")
  if(DBI::dbCanConnect(odbc::odbc(), .connection_string = con_chaine, timeout = 10, encoding = "UTF-8")){

    expect_error(connect_to_dbi(db = "test_datalibaba", user = "test", driver="Mysql"),
                 "The driver must be NULL or 'PostgreSQL'")
    expect_error(connect_to_dbi(db = "test_datalibaba", user = "test", driver=NULL),
                 "alert: Pas de driver Postgresql pour ODBC")

    expect_true(identical(class(connect_to_dbi(db = "test_datalibaba", user = "test"))[[1]], "PostgreSQL"))

    con <- DBI::dbConnect(odbc::odbc(),
                          .connection_string = con_chaine,
                          timeout = 10,
                          encoding = "UTF-8")
    expect_equal(
      str(connect_to_dbi(db = "test_datalibaba", user = "test")),
      str(con)
    )
  }else{
    print(DBI::dbCanConnect(odbc::odbc(), .connection_string = con_chaine, timeout = 10, encoding = "UTF-8"))
    print('warning: connect_to_dbi - local database postgresql doesnt work with odbc !')
    expect_equal(1,1)
  }
})
