library(DBI)
library(magrittr)
library(tibble)
library(lubridate)
library(dplyr)
library(sf)
library(glue)
library(backports)

test_that("poster_data works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    # test list_tables (utilise dans poster_data)
    schem <- paste0("test_listdata_",sample.int(1000, 1))
    condirect <- DBI::dbConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="test_datalibaba")
    DBI::dbExecute(condirect, paste0("CREATE SCHEMA ",schem," AUTHORIZATION runner;"))
    DBI::dbExecute(condirect, paste0("GRANT ALL ON SCHEMA ",schem," TO runner WITH GRANT OPTION;"))
    DBI::dbExecute(condirect, paste0("ALTER DEFAULT PRIVILEGES IN SCHEMA ",schem," GRANT ALL ON TABLES TO runner;"))
    DBI::dbExecute(condirect, paste0("ALTER DEFAULT PRIVILEGES IN SCHEMA ",schem," GRANT ALL ON SEQUENCES TO runner;"))
    expect_snapshot(list_tables(schema = schem, db = "test_datalibaba", user = "test"))
    con<<-connect_to_db(db = 'test_datalibaba', user = 'test')
    expect_snapshot(list_tables(schema = schem, con = con))
  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("poster_data works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    #tests poster_data

    expect_error(poster_data(table = "test_iris", schema = "public", db = "test_datalibaba",
                             pk = NULL, post_row_name = FALSE, overwrite = TRUE, user = "test"),
                 "'data' n\'est pas renseign\u00e9")
    expect_error(poster_data(data=c(1,2,3), table = "test_iris", schema = "public", db = "test_datalibaba",
                             pk = NULL, post_row_name = FALSE, overwrite = TRUE, user = "test"),
                 "le dataframe 'data' n\'exsiste pas")
    expect_error(poster_data(data = iris, table = "test_iris", schema = "public", db = "test_datalibaba",
                           pk = "Speciez", post_row_name = FALSE, overwrite = TRUE, user = "test"),
                 "mauvaise declaration de pk : doit etre 1 ou NULL ou un nom de variable de 'data' ou un vecteur de noms de variables de 'data'")
    expect_error(poster_data(data = iris, table = "test_iris", db = "test_datalibaba",
                             pk = NULL, post_row_name = FALSE, overwrite = TRUE, user = "test"),
                 "schema n\'est pas renseign\u00e9")
    expect_error(poster_data(data = iris, schema = "public", db = "test_datalibaba",
                             pk = NULL, post_row_name = FALSE, overwrite = TRUE, user = "test"),
                 "table n\'est pas renseign\u00e9")

  # test création schema, post_row_name, ecoSQL, pk et overwrite

    schem <- paste0("test_",sample.int(1000, 1),"_",sample.int(1000, 1))

    expect_warning(poster_data(data = iris, table = "test_iris", schema = schem, db = "test_datalibaba", pk = NULL, post_row_name = TRUE, overwrite = TRUE, user = "test"),
                   paste0("le schema ",schem," n\\'existe pas\\. Il va être créé sous test_datalibaba\\."))

    df<-iris %>% rownames_to_column("id") %>% mutate(todate = parse_date_time("2025-02-27 14:29:32", "Ymd HMS"))

    expect_message(poster_data(data = df, table = "test_iris", schema = "public", db = "test_datalibaba", pk = "id", post_row_name = TRUE, overwrite = TRUE, user = "test", ecoSQL = FALSE),
                   "\u00e9crit dans la table ")
    expect_message(poster_data(data = df, table = "test_iris", schema = "public", db = "test_datalibaba", pk = 1, post_row_name = TRUE, overwrite = TRUE, user = "test"),
                   "\u00e9crit dans la table ")
    expect_error(poster_data(data = iris, table = "test_iris", schema = schem, db = "test_datalibaba", pk = NULL, post_row_name = FALSE, overwrite = FALSE, user = "test"),
    paste0("La table test_iris existe d\u00e9j\u00e0 sur le schema ",schem," et le param\u00e8tre overwrite est \u00e0 FALSE"))

  # test geometry
    nc <- st_read(system.file("shape/nc.shp", package="sf"))
    expect_message(poster_data(data = nc, table = "nc", schema = "public", db = "test_datalibaba", pk = NULL, post_row_name = FALSE, overwrite = TRUE, user = "test"),
                   "\u00e9crit dans la table ")

  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("post_data works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))

    expect_error(post_data(con=NULL, data = iris, schema = "public", table = "test_iris", overwrite = FALSE),
                 "con n\'est pas renseign\u00e9")
    expect_error(post_data(con=connect_to_db(db = 'test_datalibaba', user = 'test'), data = iris, schema = "public", table = "test_iris", overwrite = FALSE),
                 "le connecteur con n\'existe pas dans l\'environnement")
    con<<-connect_to_db(db = 'test_datalibaba', user = 'test')
    expect_error(post_data(con=con, data = iris, schema = "public", table = "test_iris", overwrite = FALSE),
                 "veuillez utiliser une connexion avec ecoSQL=FALSE")
    con<<-connect_to_db(db = 'test_datalibaba', user = 'test', ecoSQL = FALSE)
    expect_error(post_data(con=con, schema = "public", table = "test_iris", overwrite = FALSE),
                 "data n\'est pas renseign\u00e9")
    expect_error(post_data(con=con, data=c(1), schema = "public", table = "test_iris", overwrite = FALSE),
                 glue::glue("le dataframe 1 n\'exsiste pas"))
    expect_error(post_data(con=con, data = iris, table = "test_iris", overwrite = FALSE),
                 "schema n\'est pas renseign\u00e9")
    expect_error(post_data(con=con, data = iris, schema = "public", overwrite = FALSE),
                 "table n\'est pas renseign\u00e9")
    expect_error(post_data(con=con, data = iris, schema = "titi-test", table = "test_iris", overwrite = FALSE),
                 glue::glue("le schema titi-test n\'existe pas"))

    post_data(con, data = iris, schema = "public", table = "retest_iris", overwrite = TRUE)

    expect_error(post_data(con, data = iris, schema = "public", table = "retest_iris", overwrite = FALSE),
                 paste0("La table retest_iris existe d\u00e9j\u00e0 sur le schema public et le param\u00e8 tre overwrite est \u00e0 FALSE"))
    expect_message(post_data(con, data = iris, schema = "public", table = "retest_iris", overwrite = TRUE),
                   " est \u00e9cras\u00e9e sur le schema ")
  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})

test_that("post_data_dbi works", {
  if(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres")){
    Sys.setenv(port="5432")
    Sys.setenv(server="localhost")
    Sys.setenv(user_test="runner")
    Sys.setenv(pwd_test=Sys.getenv("BDD_RUNNER_PWD"))
    con<<-connect_to_db(db = 'test_datalibaba', user = 'test', ecoSQL = FALSE)
    expect_message(suppressWarnings(post_data_dbi(con, data = iris, schema = "public", table = "retest_iris", overwrite = TRUE)),
                   " est \u00e9cras\u00e9e sur le schema ")
  }else{
    print(DBI::dbCanConnect(RPostgres::Postgres(), host="localhost", user="runner", password=Sys.getenv("BDD_RUNNER_PWD"), dbname="postgres"))
    print('warning: connect_to_db - local database postgresql doesnt work !')
    expect_equal(1,1)
  }
})
